' demonstration program
' send lines to the LCD
'
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' LCD driver for MMBasic 3.1 or later (uses defined subroutines)
' Geoff Graham, Jan 2012.
' Modified by James Deakins to work with 20 x 4 LCDs, Feb 2012.
' Should also work with 8 x 1 and other configs, but not tested.
'
' Will drive standard 16 x 2 and 20 x 4 LCDs
' For example:  futurlec.com       LCD16X2
'               altronics.com.au   Z7001
'               jaycar.com.au      QP5512
'
' To use:
'  - Define LCD size with the command:  InitLCDVariables columns, lines
'  - Setup the LCD with the command:    InitLCD
'  - Display a line using the command:  PrintLCD LineNbr, Text$
'
' File LCD_LIB.BAS will provide an extended range of functions.
'
' UMite pin 2 is RS, pin 3 is EN
' Pins 4 to 6 are D4 to D6
' Pin 16 = D7  (just because it fits my board layout)
'Use Pin 26 For PMW # 1

SetPin(2), Dout	'RS
SetPin(3), Dout	'EN

SetPin(16), Dout	'D7
SetPin(6), Dout		'D6
SetPin(5), Dout		'D5
SetPin(4), Dout		'D4


'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

PWM 1, 1350, 10		'Adjust frequency for contrast adjustment

Dim C_LineStart(4)  ' an array to hold the start addresses for each line
C_Lines = 0     ' variable to hold the number of lines on the LCD
C_LineLength = 0  ' variable to hold the line length of the LCD
'
' **** Change the parameters to indicate whether you have a 16 x 2
' **** or a 20 x 4 LCD screen.

InitLCDVariables(20, 4) ' <<<<<<<<<<-------

InitLCD

' display simple lines, based on how many lines the LCD screen has

PrintLCD 1, "    U-Mite LCD"
If C_Lines > 1 Then PrintLCD 2, "    3.3V Only"
If C_Lines > 2 Then PrintLCD 3, "   PMW Contrast"
If C_Lines > 3 Then PrintLCD 4, " Using Subroutines"
EndIf

'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' Initialise the variables for this application
' (C_Prefix indicates constants).
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Sub InitLCDVariables(LCD_LineLength, LCD_Lines)

    C_LineLength = LCD_LineLength
    C_Lines = LCD_Lines

    C_LineStart(1) = 0        ' Start address for line 1
    C_LineStart(2) = 64       ' Start address for line 2
    C_LineStart(3) = 20       ' Start address for line 3
    C_LineStart(4) = 84       ' Start address for line 4

End Sub


'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' Initialise the LCD hardware
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Sub InitLCD
'    For i = 11 To 16 : SetPin i, 9 : Next i   ' all open collector
    LCD_Nibble &B0011, 0, 5           ' reset
    LCD_Nibble &B0011, 0, 5           ' reset
    LCD_Nibble &B0011, 0, 5           ' reset
    LCD_Nibble &B0010, 0, 2           ' 4 bit mode
    LCD_Nibble &B0010 : LCD_Nibble &B1100   ' 4 bits, 2 lines
    LCD_Nibble &B0000 : LCD_Nibble &B1100   ' display on, no cursor
    LCD_Nibble &B0000 : LCD_Nibble &B0110   ' increment on write
    LCD_Nibble &B0000 : LCD_Nibble &B0001, 0, 2 ' clear the display, pause 2
End Sub

'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' Move cursor to a location on the LCD screen.
' argument #1 is the line to be used (1 to 4)
' argument #2 is the column to be used (1 to 20)
' This Subroutine isn't used by the other code, but
' is included because it's simple and useful.
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Sub LCDLocate(Line, Column)
  Local curPosn

  curPosn = 128 + C_LineStart(Line) + Column - 1
  LCD_Nibble Int(curPosn/16) : LCD_Nibble curPosn

End Sub


'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' Display a line on the LCD
' argument #1 is the line to be used (1 to 4)
' argument #2 is the line to display (can be any length)
' Note: The text has to be padded out with spaces, otherwise
' 'odd' characters are displayed on the LCD.
' The code does the padding for you.
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Sub PrintLCD ( LineNumber, Line$ )
  Local i, c, fillSpaces, fullLine$, curPosn

  curPosn = 128 + C_LineStart(LineNumber)

  ' first, send the cursor position (in two nibbles)
  LCD_Nibble Int(curPosn/16) : LCD_Nibble curPosn

  fillSpaces = C_LineLength - Len(Line$) ' is the line full?
  If fillSpaces > 0 Then
    fullLine$ = Line$ + Space$(fillSpaces)
  Else
    fullLine$ = Line$
  EndIf

  ' then send the text character by character (two nibbles per character)
  For i = 1 To C_LineLength
    c = Asc(Mid$(fullLine$, i, 1))
    LCD_Nibble Int(c/16), 1 : LCD_Nibble c, 1
  Next i

End Sub


'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' Send the lower 4 bits (called a nibble) to the LCD
' argument #1 is the nibble to send
' argument #2 is true if data, false if command (default is command)
' argument #3 is delay after the data has been sent (default is zero)
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Sub LCD_Nibble ( Data, Flag, Wait_mSec )
  Pin(2) = Flag
  Pin(4) = Data And &B00000001
  Pin(5) = Data And &B00000010
  Pin(6) = Data And &B00000100
  Pin(16) = Data And &B00001000
  Pin(3) = 1 : Pin(3) = 0
  If Wait_mSec > 0 Then Pause Wait_mSec       ' modified 30/1/2012. Restore to Wait_mSec when Pause 0 bug corrected.
End Sub
